package cn.reghao.tnb.file.app.zoss.db.mapper;

import cn.reghao.jutil.jdk.web.db.BaseMapper;
import cn.reghao.tnb.file.app.zoss.model.po.UploadChannel;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author reghao
 * @date 2025-10-16 14:13:04
 */
@Mapper
public interface UploadChannelMapper extends BaseMapper<UploadChannel> {
    void deleteById(int id);

    int countByCreateBy(long createBy);
    List<UploadChannel> findByCreateBy(long createBy);
    UploadChannel findById(int id);
    List<UploadChannel> findByCreateByAndUserNodeId(@Param("createBy") long createBy, @Param("userNodeId") int userNodeId);
    UploadChannel findByCreateByAndChannelCode(@Param("createBy") long createBy, @Param("channelCode") int channelCode);
    UploadChannel findByCreateByAndPrefix(@Param("createBy") long createBy, @Param("prefix") String prefix);
    UploadChannel findByCreateByAndName(@Param("createBy") long createBy, @Param("name") String name);
}
